// This is a program by team Possible, CultivAid (Wieger)
//It senses multiple data and sends this to Processing
//change the RGB led depend on the humidity value.
//sense humidity, temperature and soil moisture and display these values on a lcd.
//Also send these values to Processing

#include <dht.h> //temperature and humidity sensor
#include <Adafruit_GFX.h> //used for the oled display
#include <Adafruit_SSD1306.h>
#define OLED_RESET 4 //only used to create a display
Adafruit_SSD1306 display(OLED_RESET);

dht DHT;

unsigned long startMillis;
unsigned long currentMillis;
int ledPinRed = 11; //RGB pins for the RGB led
int ledPinBlue = 10;
int ledPinGreen = 9;
int ledPinRain = 2;//LED that lights up when there is rain expected
int soilPin = A1;
int humiPin = A2;

int soilmoistValue;
int humiValue;
int temperature;
int incommingValue;
bool showingTemp = true; //booleans for the lcd to switch between data
bool showingSoil = false;

void setup() {
  Serial.begin(9600);
  startMillis = millis();  //initial start time
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C); //oled display
  
  pinMode(ledPinRed, OUTPUT);
  pinMode(ledPinGreen, OUTPUT);
  pinMode(ledPinBlue, OUTPUT);
  pinMode(ledPinRain, OUTPUT);

}

void loop() {
  if (Serial.available() > 0) {
    incommingValue = Serial.read(); //read the value from processing to know if it is expected to rain

    if (incommingValue == 'Y') {
      digitalWrite(ledPinRain, HIGH); //turn led on when rain is expected
    }
    if (incommingValue == 'N') {
      digitalWrite(ledPinRain, LOW); //turn led off when no rain is expected
    }
  }

  Serial.print("H");//send to processing
  Serial.println(humiValue);
  Serial.print("T");
  Serial.println(temperature);
  Serial.print("S");
  Serial.println(soilmoistValue);

  soilmoistValue = map(analogRead(soilPin), 0, 850, 0, 100); //850 is the measured max values of the sensor
  DHT.read11(humiPin);
  humiValue = DHT.humidity;
  temperature = DHT.temperature;
  LEDcolor(map(humiValue, 100, 0, 0, 150), map(humiValue, 100, 0, 150, 0), 0); //green when very humid, but goes to yellow/red when it is not humdid

  currentMillis = millis();  //get the current "time" (actually the number of milliseconds since the program started)

  if (showingTemp) { //if statements to switch between humidity and temperature and soil moist, on the lcd.
    displayTemperature();
    if (currentMillis - startMillis >= 4000)  //test whether the period has elapsed
    {
      showingTemp = false;
      startMillis = currentMillis;  
    }
  } else if (!showingSoil) {
    displayHumidity();
    if (currentMillis - startMillis >= 4000)  //test whether the period has elapsed
    {
      showingSoil = true;
      startMillis = currentMillis;  //IMPORTANT to save the start time of the current LED state.
    }
  } else {
    displaySoil();
    if (currentMillis - startMillis >= 4000)  //test whether the period has elapsed
    {
      showingSoil = false;
      showingTemp = true;
      startMillis = currentMillis;  //IMPORTANT to save the start time of the current LED state.
    }
  }

  display.display();
  delay(1000);
}

void LEDcolor(int redLEDvalue, int greenLEDvalue, int blueLEDvalue) //function to control the RGB led more easily
{
  analogWrite(ledPinRed, redLEDvalue);
  analogWrite(ledPinGreen, greenLEDvalue);
  analogWrite(ledPinBlue, blueLEDvalue);
}

void displayTemperature() { //display temperature
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1.5);
  display.setCursor(0, 0);
  display.print("Temperature:  ");
  display.setCursor(0, 10);
  display.setTextSize(3);
  display.print(temperature);
  display.print(" C");
}

void displayHumidity() { //display humidity
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1.5);
  display.setCursor(0, 0);
  display.print("Humidity:  ");
  display.setCursor(0, 10);
  display.setTextSize(3);
  display.print(humiValue);
  display.print(" %");
}

void displaySoil() { //display soil moist
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1.5);
  display.setCursor(0, 0);
  display.print("Soil moist:  ");
  display.setCursor(0, 10);
  display.setTextSize(3);
  display.print(soilmoistValue);
  display.print(" %");
}
